import Vue from 'vue';

/*
  Normalize vuex namespace - add slashes if they are not presented
*/
var normalizeNamespace = function normalizeNamespace() {
  var namespace = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

  if (typeof namespace !== 'string') {
    namespace = '';
  }

  if (namespace.length > 0) {
    if (namespace.charAt(namespace.length - 1) !== '/') {
      namespace += '/';
    }
  }

  return namespace;
};
/*
  Copypasted code from Vuex source
  Sad, but it's not exportable:(
*/


var normalizeMap = function normalizeMap(map) {
  return Array.isArray(map) ? map.map(function (key) {
    return {
      key: key,
      val: key
    };
  }) : Object.keys(map).map(function (key) {
    return {
      key: key,
      val: map[key]
    };
  });
};
/*
  convert any name to uppercase format delimited by underscore
  Please, look at the test cases to get more info
*/


var toMutationName = function toMutationName(name) {
  var parts = name.match(/([A-Z_]?[^A-Z_]*)/g).slice(0, -1);
  return parts.join('_').toUpperCase();
};
/*
  Capitalize inputString
*/


var capitalize = function capitalize(inputString) {
  return "".concat(inputString.charAt(0).toUpperCase()).concat(inputString.slice(1).toLowerCase());
};

/*
  vuex-model
  Simple getter/action/mutation generator and getter/setter mapper for vuex

  @author: ikenfin
*/
/*
  action/mutation/getter prefix
  used to prevent collisions with user defined code
*/

var PREFIX = 'Vxm';
/*
  Generate getter/action/mutation's for vuex store
  storeAttr - is required state attribute name - this holds values
*/

var genVuexModels = function genVuexModels(fields) {
  var storeAttr = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  var result = {
    getters: {},
    actions: {},
    mutations: {},
    state: {}
  }; // early exit on falsy fields argument
  // using naive checkers here

  if (!Array.isArray(fields) && Object.prototype.toString.call(fields) !== '[object Object]') {
    throw new Error('genVuexModels: incorrect [fields] argument. Pass models array or configuration object as first argument! Passed: ', JSON.stringify(fields));
  } // we expect that if we got non-array value - then it's object


  var setDefaultValue = !Array.isArray(fields);

  if (storeAttr !== false) {
    if (!storeAttr) {
      storeAttr = PREFIX;
    }

    result.state[storeAttr] = {};
  }

  normalizeMap(fields).reduce(function (prev, _ref) {
    var key = _ref.key,
        val = _ref.val;

    if (setDefaultValue) {
      if (storeAttr) {
        result.state[storeAttr][key] = val;
      } else {
        result.state[key] = val;
      }
    }

    prev.getters["".concat(PREFIX, "_").concat(key)] = function (state) {
      return storeAttr !== false ? state[storeAttr][key] : state[key];
    };

    prev.actions["set".concat(PREFIX, "_").concat(capitalize(key))] = function (_ref2, data) {
      var commit = _ref2.commit;
      commit(toMutationName("".concat(PREFIX, "_").concat(key)), data);
    };

    prev.mutations[toMutationName("".concat(PREFIX, "_").concat(key))] = function (state, value) {
      storeAttr !== false ? Vue.set(state[storeAttr], key, value) : Vue.set(state, key, value);
    };

    return prev;
  }, result);
  return result;
};
/*
  Create { actions, getters, mutations } object to be injected in components
*/


var mapVuexModels = function mapVuexModels(models) {
  var namespace = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  namespace = normalizeNamespace(namespace);
  models = normalizeMap(models);
  return models.reduce(function (prev, _ref3) {
    var key = _ref3.key,
        val = _ref3.val;
    prev[key] = {
      get: function get() {
        return this.$store.getters["".concat(namespace).concat(PREFIX, "_").concat(val)];
      },
      set: function set(value) {
        this.$store.dispatch("".concat(namespace, "set").concat(PREFIX, "_").concat(capitalize(val)), value);
      }
    };
    return prev;
  }, {});
};

export { genVuexModels, mapVuexModels };
//# sourceMappingURL=bundle.esm.js.map
